@@ -7,7 +7,7 @@ from copy import deepcopy
 from typing import TYPE_CHECKING, Literal
 
 from datasets import Dataset
-from openai import AsyncOpenAI, OpenAI
+from openai import AsyncOpenAI, BadRequestError, OpenAI
 
 from verifiers.parsers.parser import Parser
 from verifiers.rubrics.rubric import Rubric
@@ -27,6 +27,11 @@ from verifiers.types import (
     SamplingArgs,
     State,
 )
+from verifiers.utils.message_utils import (
+    cleanup_messages,
+    get_overlong_prompt_dummy_response,
+    sanitize_tool_calls,
+)
 
 if TYPE_CHECKING:
     from transformers.tokenization_utils_base import (  # type: ignore
@@ -41,8 +46,6 @@ class Environment(ABC):
 
     def __init__(
         self,
-        client: AsyncOpenAI | None = None,
-        model: str | None = None,
         dataset: Dataset | None = None,
         eval_dataset: Dataset | None = None,
         system_prompt: str | None = None,
@@ -55,14 +58,17 @@ class Environment(ABC):
         max_workers: int = 512,
         **kwargs,
     ):
-        self.client = client
-        self.model = model
+        self.logger = logging.getLogger(f"verifiers.envs.{self.__class__.__name__}")
         self.message_type: Literal["chat", "completion"] = message_type
         self.oai_tools: list[ChatCompletionToolParam] | None = oai_tools
         self.system_prompt = system_prompt
         self.few_shot = few_shot
         self.parser = parser or Parser()
         self.rubric = rubric or Rubric()
+        if self.parser.__class__ != self.rubric.parser.__class__:
+            self.logger.warning(
+                "The parser and rubric parser are different. This may cause unexpected behavior."
+            )
 
         if self.message_type == "chat":
             if dataset is not None:
@@ -82,7 +88,7 @@ class Environment(ABC):
                 raise ValueError(
                     'The fields "system_prompt" and "few_shot" are not supported for completion tasks.'
                     'Please use message_type="chat" instead, or pre-format your dataset '
-                    'to contain "prompt" and "answer" columns.'
+                    'to contain a "prompt" column.'
                 )
             self.dataset = dataset
             self.eval_dataset = eval_dataset
@@ -97,7 +103,6 @@ class Environment(ABC):
                     self.sampling_args[key] = value
 
         self.max_workers = max_workers
-        self.logger = logging.getLogger(f"verifiers.envs.{self.__class__.__name__}")
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -127,8 +132,8 @@ class Environment(ABC):
         answer_key: str = "answer",
     ) -> Dataset:
         # skip if "prompt" already exists
-        if "prompt" in dataset.column_names:
-            return dataset
+        if "id" not in dataset.column_names:
+            dataset = dataset.add_column("id", range(len(dataset)))  # type: ignore
 
         # extract format_prompt as a standalone function to avoid capturing self
         def format_prompt_fn(prompt_str: str) -> list[ChatMessage]:
@@ -140,19 +145,23 @@ class Environment(ABC):
             messages.append({"role": "user", "content": prompt_str})
             return messages
 
-        if answer_key == "answer":
-            return dataset.map(
-                lambda x: {
-                    "prompt": format_prompt_fn(x[question_key]),
-                }
-            )
-        else:
-            return dataset.map(
-                lambda x: {
-                    "prompt": format_prompt_fn(x[question_key]),
-                    "answer": x[answer_key],
-                }
-            )
+        if "prompt" not in dataset.column_names:
+            if answer_key == "answer":
+                dataset = dataset.map(
+                    lambda x: {
+                        "prompt": format_prompt_fn(x[question_key]),
+                    }
+                )
+            else:
+                dataset = dataset.map(
+                    lambda x: {
+                        "prompt": format_prompt_fn(x[question_key]),
+                        "answer": x[answer_key],
+                    }
+                )
+        assert "id" in dataset.column_names
+        assert "prompt" in dataset.column_names
+        return dataset
 
     def get_dataset(self, n: int = -1, seed: int | None = None) -> Dataset:
         if self.dataset is None:
@@ -160,6 +169,8 @@ class Environment(ABC):
         if seed is not None:
             self.dataset = self.dataset.shuffle(seed=seed)
         if n > 0:
+            # Cap n to the length of the dataset to prevent IndexError
+            n = min(n, len(self.dataset))
             return self.dataset.select(range(n))
         return self.dataset
 
@@ -172,6 +183,8 @@ class Environment(ABC):
         if seed is not None:
             self.eval_dataset = self.eval_dataset.shuffle(seed=seed)
         if n > 0:
+            # Cap n to the length of the dataset to prevent IndexError
+            n = min(n, len(self.eval_dataset))
             return self.eval_dataset.select(range(n))
         return self.eval_dataset
 
@@ -198,27 +211,60 @@ class Environment(ABC):
         Returns special error messages for context length issues.
         """
         sampling_args = sampling_args or {}
+        # Resolve message type first
+        if message_type is None:
+            message_type = self.message_type
+        # Normalize sampling args:
+        # - If max_tokens is provided for chat, rename to max_completion_tokens
+        # - Drop any None-valued entries to avoid sending them to the client
         if "max_tokens" in sampling_args:
-            sampling_args["max_completion_tokens"] = sampling_args.pop("max_tokens")
-
+            if sampling_args["max_tokens"] is None:
+                sampling_args.pop("max_tokens")
+            elif message_type == "chat":
+                sampling_args["max_completion_tokens"] = sampling_args.pop("max_tokens")
+        if (
+            "max_completion_tokens" in sampling_args
+            and sampling_args["max_completion_tokens"] is None
+        ):
+            sampling_args.pop("max_completion_tokens")
+        clean_sampling_args = {k: v for k, v in sampling_args.items() if v is not None}
         try:
-            if message_type is None:
-                message_type = self.message_type
-
             if message_type == "chat":
                 assert isinstance(prompt, list)
+                # --- detect audio parts and force text-only modality if caller didn't set one ---
+                has_audio = False
+                try:
+                    for m in prompt:
+                        c = m.get("content")  # type: ignore[assignment]
+                        if isinstance(c, list):
+                            for p in c:
+                                if isinstance(p, dict) and str(
+                                    p.get("type", "")
+                                ).startswith("input_audio"):
+                                    has_audio = True
+                                    break
+                        if has_audio:
+                            break
+                except Exception:
+                    has_audio = False
+                if has_audio and "modalities" not in clean_sampling_args:
+                    clean_sampling_args = {
+                        **clean_sampling_args,
+                        "modalities": ["text"],
+                    }
+
                 if oai_tools:
                     response = await client.chat.completions.create(
                         model=model,
                         messages=prompt,  # type: ignore
                         tools=oai_tools,
-                        **sampling_args,
+                        **clean_sampling_args,
                     )
                 else:
                     response = await client.chat.completions.create(
                         model=model,
                         messages=prompt,  # type: ignore
-                        **sampling_args,
+                        **clean_sampling_args,
                     )
                 return response
             elif message_type == "completion":
@@ -228,10 +274,19 @@ class Environment(ABC):
                     )
                 assert isinstance(prompt, str)
                 response = await client.completions.create(
-                    model=model, prompt=prompt, **sampling_args
+                    model=model, prompt=prompt, **clean_sampling_args
                 )
                 return response
         except Exception as e:
+            # In case of making a request with an overlong prompt, e.g from a too-long
+            # environment response, we return a dummy response to with finish_reason "length"
+            if isinstance(e, BadRequestError) and e.response.text.startswith(
+                '{"error":{"message":"This model\'s maximum context length is'
+            ):
+                self.logger.debug("Caught overlong prompt.")
+                return get_overlong_prompt_dummy_response(
+                    message_type or self.message_type
+                )
             self.logger.error(f"Error getting model response: {e} \n\nExiting...")
             raise e
 
@@ -320,11 +375,14 @@ class Environment(ABC):
     async def a_generate(
         self,
         inputs: GenerateInputs | Dataset | dict,
-        client: AsyncOpenAI | None = None,
-        model: str | None = None,
+        client: AsyncOpenAI,
+        model: str,
         sampling_args: SamplingArgs | None = None,
         score_rollouts: bool = True,
         max_concurrent: int = -1,
+        max_concurrent_generation: int | None = None,
+        max_concurrent_scoring: int | None = None,
+        interleave_scoring: bool = True,
         **kwargs,
     ) -> GenerateOutputs:
         """
@@ -332,13 +390,6 @@ class Environment(ABC):
         """
         if isinstance(inputs, GenerateInputs):
             inputs = inputs.model_dump()
-        # use class-level client and model if not provided
-        if client is None:
-            assert self.client is not None
-            client = self.client
-        if model is None:
-            assert self.model is not None
-            model = self.model
         gen_sampling_args = deepcopy(self.sampling_args)
         if sampling_args is not None:
             gen_sampling_args.update(sampling_args)
@@ -351,7 +402,10 @@ class Environment(ABC):
             for col in inputs.column_names:
                 if col == "info":
                     # handle info column to ensure mutable dicts
-                    results_dict[col] = [dict(item) for item in inputs[col]]
+                    if isinstance(inputs[col][0], str):
+                        results_dict[col] = [json.loads(item) for item in inputs[col]]
+                    else:
+                        results_dict[col] = [dict(item) for item in inputs[col]]
                 else:
                     results_dict[col] = deepcopy(inputs[col])
         else:
@@ -359,12 +413,17 @@ class Environment(ABC):
         if "prompt" not in results_dict:
             raise ValueError("prompt column not found in inputs")
         if "answer" not in results_dict and "info" not in results_dict:
-            raise ValueError("answer or info column must be found in inputs")
-        if "answer" not in results_dict:
+            self.logger.warning(
+                "Neither 'answer' nor 'info' column found in inputs. "
+                "Some environments can evaluate using only prompt/completion/state, "
+                "but reward functions requiring ground truth data may return 0.0. "
+                "Proceeding with empty values."
+            )
+        if not results_dict.get("answer"):
             results_dict["answer"] = [""] * len(results_dict["prompt"])
-        if "task" not in results_dict:
+        if not results_dict.get("task"):
             results_dict["task"] = ["default"] * len(results_dict["prompt"])
-        if "info" not in results_dict:
+        if not results_dict.get("info"):
             results_dict["info"] = [{}] * len(results_dict["prompt"])
         for i, info in enumerate(results_dict["info"]):
             if isinstance(info, str):
@@ -372,6 +431,8 @@ class Environment(ABC):
             if self.oai_tools and "oai_tools" not in info:
                 info["oai_tools"] = self.oai_tools
 
+        results_dict["prompt"] = [cleanup_messages(p) for p in results_dict["prompt"]]
+
         # prepare GenerateOutputs and run rollouts
         results = GenerateOutputs(
             prompt=results_dict["prompt"],
@@ -383,41 +444,151 @@ class Environment(ABC):
             reward=[],
             metrics={},
         )
-        rollouts = await self.run_rollouts(
-            prompts=results.prompt,
-            answers=results.answer,
-            tasks=results.task,
-            infos=results.info,
-            client=client,
-            model=model,
-            sampling_args=gen_sampling_args,
-            max_concurrent=max_concurrent,
-            **kwargs,
-        )
-        results.completion = [rollout[0] for rollout in rollouts]
-        results.state = [rollout[1] for rollout in rollouts]
-        if score_rollouts:
-            rollout_scores = await self.rubric.score_rollouts(
+        n = len(results.prompt)
+
+        # Resolve concurrency knobs
+        gen_limit = max_concurrent_generation
+        score_limit = max_concurrent_scoring
+        if gen_limit is None:
+            gen_limit = max_concurrent
+        if score_limit is None:
+            score_limit = max_concurrent
+
+        if interleave_scoring and score_rollouts:
+            # Interleaved pipeline: generate and score per rollout with separate semaphores
+            results_completion: list[Messages] = [None] * n  # type: ignore[assignment]
+            results_state: list[State] = [None] * n  # type: ignore[assignment]
+            rewards: list[float] = [0.0] * n
+            # Pre-allocate metrics using known reward function names
+            reward_func_names = self.rubric.get_reward_func_names()
+            metrics: dict[str, list[float]] = {
+                name: [0.0] * n for name in reward_func_names
+            }
+
+            gen_semaphore = (
+                asyncio.Semaphore(gen_limit) if gen_limit and gen_limit > 0 else None
+            )
+            score_semaphore = (
+                asyncio.Semaphore(score_limit)
+                if score_limit and score_limit > 0
+                else None
+            )
+
+            async def run_one(i: int) -> None:
+                prompt_i = results.prompt[i]
+                answer_i = results.answer[i]
+                task_i = results.task[i]
+                info_i = results.info[i]
+                # Generation stage
+                if gen_semaphore is not None:
+                    async with gen_semaphore:
+                        comp_i, state_i = await self.rollout(
+                            client,
+                            model,
+                            prompt_i,
+                            answer_i,
+                            task_i,
+                            info_i,
+                            gen_sampling_args,
+                            **kwargs,
+                        )
+                else:
+                    comp_i, state_i = await self.rollout(
+                        client,
+                        model,
+                        prompt_i,
+                        answer_i,
+                        task_i,
+                        info_i,
+                        gen_sampling_args,
+                        **kwargs,
+                    )
+                results_completion[i] = comp_i
+                results_state[i] = state_i
+                # Scoring stage
+                if score_semaphore is not None:
+                    async with score_semaphore:
+                        rs = await self.rubric.score_rollout(
+                            prompt=prompt_i,
+                            completion=comp_i,
+                            answer=answer_i,
+                            state=state_i,
+                            task=task_i,
+                            info=info_i,
+                            **kwargs,
+                        )
+                else:
+                    rs = await self.rubric.score_rollout(
+                        prompt=prompt_i,
+                        completion=comp_i,
+                        answer=answer_i,
+                        state=state_i,
+                        task=task_i,
+                        info=info_i,
+                        **kwargs,
+                    )
+                rewards[i] = rs.reward
+                for k, v in rs.metrics.items():
+                    # Ensure key exists in case of EnvGroup/RubricGroup dynamics
+                    if k not in metrics:
+                        metrics[k] = [0.0] * n
+                    metrics[k][i] = v
+
+            tasks = [run_one(i) for i in range(n)]
+            from tqdm.asyncio import tqdm_asyncio
+
+            await tqdm_asyncio.gather(
+                *tasks, total=n, desc=f"Running {n} rollouts (interleaved)"
+            )
+
+            results.completion = results_completion  # type: ignore[assignment]
+            results.state = results_state  # type: ignore[assignment]
+            results.reward = rewards
+            results.metrics = metrics
+            return results
+        else:
+            # Non-interleaved: generate all then score all
+            rollouts = await self.run_rollouts(
                 prompts=results.prompt,
-                completions=results.completion,
                 answers=results.answer,
-                states=results.state,
                 tasks=results.task,
                 infos=results.info,
-                apply_weights=True,
+                client=client,
+                model=model,
+                sampling_args=gen_sampling_args,
+                max_concurrent=gen_limit if gen_limit is not None else max_concurrent,
+                **kwargs,
             )
-            results.reward = rollout_scores.reward
-            results.metrics = rollout_scores.metrics
-        return results
+            results.completion = [rollout[0] for rollout in rollouts]
+            results.state = [rollout[1] for rollout in rollouts]
+            if score_rollouts:
+                rollout_scores = await self.rubric.score_rollouts(
+                    prompts=results.prompt,
+                    completions=results.completion,
+                    answers=results.answer,
+                    states=results.state,
+                    tasks=results.task,
+                    infos=results.info,
+                    max_concurrent=score_limit
+                    if score_limit is not None
+                    else max_concurrent,
+                    apply_weights=True,
+                )
+                results.reward = rollout_scores.reward
+                results.metrics = rollout_scores.metrics
+            return results
 
     def generate(
         self,
         inputs: GenerateInputs | Dataset,
         client: AsyncOpenAI | OpenAI,
-        model: str | None = None,
+        model: str,
         sampling_args: SamplingArgs | None = None,
         score_rollouts: bool = True,
         max_concurrent: int = -1,
+        max_concurrent_generation: int | None = None,
+        max_concurrent_scoring: int | None = None,
+        interleave_scoring: bool = True,
         **kwargs,
     ) -> GenerateOutputs:
         if isinstance(client, OpenAI):
@@ -429,6 +600,9 @@ class Environment(ABC):
             sampling_args,
             score_rollouts,
             max_concurrent,
+            max_concurrent_generation,
+            max_concurrent_scoring,
+            interleave_scoring,
             **kwargs,
         )
 
@@ -455,6 +629,126 @@ class Environment(ABC):
             # shutdown the executor to prevent thread leaks
             executor.shutdown(wait=False)
 
+    #########################################################
+    # Helper functions for evaluation and dataset generation
+    #########################################################
+
+    def evaluate(
+        self,
+        client: AsyncOpenAI | OpenAI,
+        model: str,
+        sampling_args: SamplingArgs | None = None,
+        num_examples: int = -1,
+        rollouts_per_example: int = 1,
+        score_rollouts: bool = True,
+        max_concurrent: int = -1,
+        max_concurrent_generation: int | None = None,
+        max_concurrent_scoring: int | None = None,
+        interleave_scoring: bool = True,
+        **kwargs,
+    ) -> GenerateOutputs:
+        """
+        Evaluate model on the Environment evaluation dataset.
+        """
+        if self.eval_dataset is None:
+            self.logger.info("eval_dataset is not set, falling back to train dataset")
+            assert self.dataset is not None
+            inputs = self.get_dataset(n=num_examples)
+        else:
+            inputs = self.get_eval_dataset(n=num_examples)
+        assert inputs is not None, "No dataset found"
+        if rollouts_per_example > 1:
+            inputs = inputs.repeat(rollouts_per_example)
+        results = self.generate(
+            inputs,
+            client,
+            model,
+            sampling_args,
+            score_rollouts,
+            max_concurrent,
+            max_concurrent_generation,
+            max_concurrent_scoring,
+            interleave_scoring,
+            **kwargs,
+        )
+        return results
+
+    def make_dataset(
+        self,
+        results: GenerateOutputs,
+        rollouts_per_example: int = 1,
+        push_to_hf_hub: bool = False,
+        hub_name: str | None = None,
+        state_columns: list[str] | None = None,
+        **kwargs,
+    ) -> Dataset:
+        """
+        Make a dataset from the evaluation results.
+
+        Args:
+            results: The evaluation results to convert to a dataset
+            rollouts_per_example: The number of rollouts per example
+            push_to_hf_hub: Whether to push the dataset to the Hugging Face Hub
+            hub_name: The name of the dataset on the Hugging Face Hub
+            state_columns: List of state columns to include in the dataset
+            concatenate_safe: Whether to ensure the dataset can be concatenated with others
+                          by standardizing column types (useful for combining results from
+                          different environments). Defaults to True for consistent schemas.
+            **kwargs: Additional arguments passed to Dataset creation
+        """
+        # TODO: enable saving of multimodal datasets
+        state_columns = state_columns or []
+
+        if push_to_hf_hub and hub_name is None:
+            raise ValueError("hub_name must be provided if push_to_hf_hub is True")
+
+        cols = ["prompt", "completion", "answer", "task", "reward", "average_reward"]
+
+        if rollouts_per_example > 1:
+            average_reward = []
+            for i in range(0, len(results.reward), rollouts_per_example):
+                chunk = results.reward[i : i + rollouts_per_example]
+                avg = sum(chunk) / len(chunk)
+                average_reward.extend([avg] * rollouts_per_example)
+        else:
+            average_reward = results.reward
+
+        results_dict = {
+            "prompt": results.prompt,
+            "completion": [],
+            "answer": results.answer,
+            "task": results.task,
+            "reward": results.reward,
+            "average_reward": average_reward,
+        }
+        if results.info[0] != {}:
+            results_dict["info"] = results.info
+            cols.append("info")
+        for i in range(len(results.completion)):
+            results_dict["completion"].append(
+                sanitize_tool_calls(results.completion[i])
+            )
+        results_dict.update(results.metrics)
+        cols.extend(results.metrics.keys())
+        if results.state[0] is not None:
+            for col in state_columns:
+                if col in results.state[0]:
+                    results_dict[col] = [state[col] for state in results.state]
+                    cols.append(col)
+                else:
+                    self.logger.warning(
+                        f"Column {col} not found in state, skipping from dataset."
+                    )
+        dataset = Dataset.from_dict({col: results_dict[col] for col in cols})
+        if push_to_hf_hub:
+            assert hub_name is not None
+            dataset.push_to_hub(hub_name)
+        return dataset
+
+    #########################################################
+    # Optional helper functions for parsing vLLM completions
+    #########################################################
+
     def parse_chat_completion_logprobs(
         self, chat_completion: ChatCompletion
     ) -> list[float]:
@@ -546,7 +840,7 @@ class Environment(ABC):
             conversation=prompt,  # type: ignore
             add_generation_prompt=True,
         )
-        messages_consumed = deepcopy(prompt)
+        messages_consumed = [m for m in prompt]
         prompt_mask: list[int] = [0] * len(prompt_ids)
         completion_ids: list[int] = []
         completion_mask: list[int] = []
@@ -554,6 +848,38 @@ class Environment(ABC):
         i = 0
         while i < len(zipped):
             message, response = zipped[i]
+
+            def deserialize_tool_calls(message: dict) -> dict:
+                """
+                Deserialize tool calls in messages, if any are present. Iterates
+                over all messages in a message list and tries to find
+                "tool_calls" key. If found, assumes it is a OAI format and has
+                key "function" with "arguments" key which is stringified. It
+                will then deserialize the argument so that chat tmeplates like
+                Qwen3's can be used.
+                """
+
+                def deserialize_tool_call(tool_call) -> dict:
+                    tool_call = dict(tool_call)
+                    function = dict(tool_call["function"])
+                    return {
+                        **tool_call,
+                        "function": {
+                            **function,
+                            "arguments": json.loads(function["arguments"]),
+                        },
+                    }
+
+                return {
+                    **message,
+                    "tool_calls": [
+                        deserialize_tool_call(tool_call)
+                        for tool_call in message.get("tool_calls", []) or []
+                    ],
+                }
+
+            message = deserialize_tool_calls(message)
+
             # assistant case -- use response
             if message["role"] == "assistant":
                 assert response is not None, "Response should not be None"
@@ -694,9 +1020,7 @@ class Environment(ABC):
         """
         Process results with vLLM tokens/logprobs.
         """
-        # Determine format from first prompt
-        # TODO: why not from self.message_type?
-        is_chat_format = isinstance(prompts[0], list)
+        is_chat_format = self.message_type == "chat"
 
         all_prompt_ids = []
         all_prompt_masks = []
@@ -734,6 +1058,7 @@ class Environment(ABC):
             if max_seq_len > 0 and len(prompt_ids) + len(completion_ids) > max_seq_len:
                 if len(prompt_ids) > max_seq_len:
                     prompt_ids = prompt_ids[:max_seq_len]
+                    prompt_mask = prompt_mask[:max_seq_len]
                 completion_ids = completion_ids[: max_seq_len - len(prompt_ids)]
                 completion_mask = completion_mask[: max_seq_len - len(prompt_ids)]
                 completion_logprobs = completion_logprobs[
@@ -773,107 +1098,3 @@ class Environment(ABC):
 
     # alias for process_env_results_vllm
     process_env_results = process_env_results_vllm
-
-    # Evaluation and dataset generation
-    def evaluate(
-        self,
-        client: AsyncOpenAI | OpenAI,
-        model: str,
-        sampling_args: SamplingArgs | None = None,
-        num_examples: int = -1,
-        rollouts_per_example: int = 1,
-        score_rollouts: bool = True,
-        max_concurrent: int = -1,
-        **kwargs,
-    ) -> GenerateOutputs:
-        """
-        Evaluate model on the Environment evaluation dataset.
-        """
-        if self.eval_dataset is None:
-            self.logger.info("eval_dataset is not set, falling back to train dataset")
-            assert self.dataset is not None
-            inputs = self.get_dataset(n=num_examples)
-        else:
-            inputs = self.get_eval_dataset(n=num_examples)
-        assert inputs is not None, "No dataset found"
-        if rollouts_per_example > 1:
-            inputs = inputs.repeat(rollouts_per_example)
-        results = self.generate(
-            inputs,
-            client,
-            model,
-            sampling_args,
-            score_rollouts,
-            max_concurrent,
-            **kwargs,
-        )
-        return results
-
-    def _sanitize_tool_calls(self, completion: Messages) -> Messages:
-        """
-        Sanitize tool calls from a completion.
-        """
-
-        assert isinstance(completion, list)
-        sanitized_completion = []
-        for m in completion:
-            if "tool_calls" in m:
-                new_m = {
-                    "role": m["role"],
-                    "content": m.get("content", ""),
-                    "tool_calls": [
-                        json.dumps(tc.model_dump())  # type: ignore
-                        for tc in m.get("tool_calls", [])
-                    ],
-                }
-                sanitized_completion.append(new_m)
-            else:
-                sanitized_completion.append(m)
-        return sanitized_completion
-
-    def make_dataset(
-        self,
-        results: GenerateOutputs,
-        push_to_hub: bool = False,
-        hub_name: str | None = None,
-        state_columns: list[str] | None = None,
-        **kwargs,
-    ) -> Dataset:
-        """
-        Make a dataset from the evaluation results.
-        """
-        state_columns = state_columns or []
-
-        if push_to_hub and hub_name is None:
-            raise ValueError("hub_name must be provided if push_to_hub is True")
-
-        cols = ["prompt", "completion", "answer", "info", "task", "reward"]
-
-        results_dict = {
-            "prompt": results.prompt,
-            "completion": [],
-            "answer": results.answer,
-            "info": results.info,
-            "task": results.task,
-            "reward": results.reward,
-        }
-        for i in range(len(results.completion)):
-            results_dict["completion"].append(
-                self._sanitize_tool_calls(results.completion[i])
-            )
-        results_dict.update(results.metrics)
-        cols.extend(results.metrics.keys())
-        if results.state[0] is not None:
-            for col in state_columns:
-                if col in results.state[0]:
-                    results_dict[col] = [state[col] for state in results.state]
-                    cols.append(col)
-                else:
-                    self.logger.warning(
-                        f"Column {col} not found in state, skipping from dataset."
-                    )
-        dataset = Dataset.from_dict({col: results_dict[col] for col in cols})
-        if push_to_hub:
-            assert hub_name is not None
-            dataset.push_to_hub(hub_name)
-        return dataset
