@@ -7,10 +7,10 @@ class SingleTurnEnv(MultiTurnEnv):
     Environment for single-turn tasks (chat or completion).
     """
 
-    def is_completed(self, messages: Messages, state: State, **kwargs) -> bool:
+    async def is_completed(self, messages: Messages, state: State, **kwargs) -> bool:
         return len(state["responses"]) > 0
 
-    def env_response(
+    async def env_response(
         self, messages: Messages, state: State, **kwargs
     ) -> tuple[Messages, State]:
         # never called in MultiTurnEnv.rollout
